// 注文情報を表すエンティティ
public class Order implements Cloneable {
  private final int id;
  private String name;
  private List<OrderDetail> orderDetails =
    new ArrayList<OrderDetail>();
  public int getId() {
    return id;
  }
  public String getName() {
    return name;
  }
  // 構築が単純である場合はコンストラクタでよい
  public Order(int id, String name) {
    this.id = id;
    this.name = name;
  }
  // OrderDetailを生成するためのファクトリメソッド
  public OrderDetail newOrderDetail(String productName,
      int price) {
    OrderDetail od = new OrderDetail(orderDetails.size(),
      productName, price);
    orderDetails.add(od);
    return od.clone();
  }
  // すべての注文明細を返す
  public List<OrderDetail> getOrderDetails() {
    // orderDetailsは可変クラスなのでコピーを作る
    List<OrderDetail> result = new ArrayList<OrderDetail>();
    for (OrderDetail od : orderDetails) {
      // 要素のOrderDetailも可変クラスなのでコピーを作る
      result.add(od.clone());
    }
    return result;
  }
  // 注文合計金額を返す
  public int getTotalPrice() {
    int result = 0;
    for (OrderDetail od : orderDetails) {
      result += od.getPrice();
    }
    return result;
  }
  // このエンティティ自体の複製を返す
  public Order clone() {
    try {
      Order result = (Order) super.clone();
      result.orderDetails = getOrderDetails();
      return result;
    } catch (CloneNotSupportedException e) {
      throw new Error(e);
    }
  }
  // E方式のequals, hashCode
}
// 注文明細を表すローカルエンティティ
public class OrderDetail implements Cloneable {
  private final int no;
  private String productName;
  private int price;
  // 不変条件を維持するためにコンストラクタを
  // パッケージプライベートにする
  OrderDetail(int no, String produceName, int price) {
    this.no = no;
    this.productName = productName;
    this.price = price;
  }
  // E方式のequals, hashCode, clone, getter/setter略
}